% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:          Script file for Bayesian analysis of VARs 
%
% AUTHOR:           Mattias Villani, Research Department, Sveriges Riksbank and
%                   Department of Statistics, Stockholm University. 
%                   E-mail: mattias.villani@riksbank.se
%
% FIRST VERSION:    2003-10-10   
% LAST REVISED:     2007-09-23
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%  Input dialog for Excel file and reading Excel sheets %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ExcelFileName = uigetfile('*.xls', 'BayesVAR - Select an Excel Workbook with Data and Settings');
disp(['Reading input from file: ',ExcelFileName])
[GeneralInfo,ExcelSheets]=xlsfinfo(ExcelFileName); % Extract names of Excel sheets.
BVARReadExcel % Script for reading all model info from the Excel file 'ExcelFileName' as defined above.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NForecastPeriods=length(estendyear1+(estendperiod1-1)/season:(1/season):estendyear2+(estendperiod2-1)/season); % Number of forecast dates.
% NBackObs is the number of past observations in the RecentMean forecast. Can be different for each forecast date.
if length(NBackObs)==1
    NBackObs=NBackObs*ones(NForecastPeriods,1);
end
if length(NBackObs)~=NForecastPeriods
    errordlg('The number of NBackObs is different from 1 and the number of forecast evaluation periods','Error in BayesianVAR')
end

% The code can re-do the estimation and forecasting for many different
% model configurations/settings, e.g. different lag lengths. 
OrigOutFileName=OutFileName;
if isempty(LoopValues)
    LoopValues=1;
    LoopQuantity='none';
end
if strcmpi(LoopQuantity,'none')
    LoopValues=1;
end
for LoopCount=1:length(LoopValues)
    OutFileName=[OrigOutFileName,'_',LoopQuantity,'_',num2str(LoopValues(LoopCount)),'_'];
    switch LoopQuantity
        case 'LagLength'
            K=LoopValues(LoopCount);
        case 'lambda1'
            lambdas(1)=LoopValues(LoopCount);
        case 'lambda2'
            lambdas(2)=LoopValues(LoopCount);
        case 'lambda3'
            lambdas(3)=LoopValues(LoopCount);
        case 'lambda4'
            lambdas(4)=LoopValues(LoopCount);
        case 'lambda5'
            lambdas(5)=LoopValues(LoopCount);
        case 'lambda6'
            lambdas(6)=LoopValues(LoopCount);
        case 'IncludeConstant'
            IncludeConstant=LoopValues(LoopCount)
        otherwise 
            OutFileName=OrigOutFileName;
    end
    
    if strcmpi(LoopQuantity,'none')
        disp('No looping over model settings.')
    else
        disp(['Looping:  ',LoopQuantity,'=',num2str(LoopValues(LoopCount))])
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    SETTING UP ESTIMATION SAMPLE PHASE    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    estendyear=estendyear1;
    estendperiod=estendperiod1;
    upperperiod=season;
    clear PredsFixedHorizon DataFixedHorizon MSE MedianYForecastAll
    
    %%%%%%%%%%%%% Initialization %%%%%%%%%%%%%%%
    MeanUnCondMean=[];
    MedianYForecast=[];
    MeanYForecast=[];
    CovYForecast=[];
    EvalHorizons=[];
    ModelLabels=[];
    RMSE=[];
    MAE=[];
    MAPE=[];
    MPE=[];
    MSSE=[];
    LogDetMSSE=[];
    TraceMSSE=[];
    MeanLogPredScore=[];
    PostProbCoverage=[];
    BandCoverageProbs=[];
    MSSEScaleMatrix=[];
    PredsFixedHorizon=[];
    DataFixedHorizon=[];

    if ML==1 % No error bands if ML is used.
        CoverProb=[];
    end
    
    NCoverProb=length(CoverProb);
    if NCoverProb==0
        ComputeBandCoverage=0;
        ComputeLogPredScore=0;
    end
    
    %%%%  Checking user inputs  -  and correcting if wrong 
    if max(FEVDHorizon)>IRHorizon
        disp('Invalid Input: Maximal horizon for forecast error variance decompositions exceeds IR horizon. Redefining FEVDHorizon.')
        FEVDHorizon=FEVDHorizon(FEVDHorizon<=IRHorizon);
    end
    
    BadInputFlag=0;
    for u=1:length(MSSESubsetVars)
        if isempty(setdiff(MSSESubsetVars{u},variables))==0 % At least one variable in MSSESubsetVars is outside the set of endogenous variables.
            BadInputFlag=1;
        end
    end
    if BadInputFlag
            disp('Invalid Input: At least one variable in MSSESubsetVars is outside the set of endogenous variables. MSSESubsetVars is redefined.')
            MSSESubsetVars=cell(0);
            MSSESubsetVars{1}=variables;
    end
    
    if ~isnan(HistDecompShocks{1})
        BadInputFlag=0;
        for u=1:length(HistDecompShocks)
            if ~isempty(setdiff(HistDecompShocks{u},variables))
                BadInputFlag=1;
            end
        end
        if BadInputFlag
            disp('Invalid Input: The indicies in HistDecompShocks must be <= # endogenous variables. HistDecompShocks is set to empty.')
            HistDecompShocks=cell(0);
            HistDecompShocks{1}=nan;
        end
    end
    
    % Redefining references to variables so that they becomes relative to PredVars
    for h=1:length(MSSESubsetVars)
        MSSESubsetVars{h}=find(ismember(variables,MSSESubsetVars{h}));
    end

    if ~isempty(setdiff(PredPlotVars,variables))
        disp('Warning: You have requested to plot the forecast of a non-modelled variable. Check the Excel sheet.')
    end
    PredPlotVars=find(ismember(variables,PredPlotVars));
    
    if ~isempty(setdiff(residualplotvars,variables))
        disp('Warning: You have requested to plot the residuals/fit of a non-modelled variable. Check the Excel sheet.')
    end
    residualplotvars=find(ismember(variables,residualplotvars));

    for h=1:length(CondShock)
        CondShock{h}=find(ismember(variables,CondShock{h}));
    end

    for h=1:length(CondVar)
        CondVar{h}=find(ismember(variables,CondVar{h}));
    end
    
    for h=1:length(HistDecompShocks)
        HistDecompShocks{h}=find(ismember(variables,HistDecompShocks{h}));
    end

    if ~isempty(setdiff(DeCompVars,variables))
        disp('Warning: You have requested to plot a historical decomposition of a non-modelled variable. Check the Excel sheet.')
    end
    DeCompVars=find(ismember(variables,DeCompVars));
    
    ShockLabels=ShockLabels(variables);   
    Grest=Grest(variables,variables);
    
    if ~isempty(setdiff(CascadePlotVars,variables))
        disp('Warning: You have requested to plot the forecast cascade of a non-modelled variable. Check the Excel sheet.')
    end
    CascadePlotVars=find(ismember(variables,CascadePlotVars));
    
    
    % Switching a few things off during the sequential forecasts. They are
    % later switched on at the final forecast date.
    if SwitchOffDuringSequential
        IRHorizonOrig=IRHorizon;
        FEVDHorizonOrig=FEVDHorizon;
        actual_fittedOrig=actual_fitted;
        crossplotOrig=crossplot;
        actualresOrig=actualres;
        autocorrresOrig=autocorrres;
        historesOrig=histores;
        SaveParametersOrig=SaveParameters;
        estsumOrig=estsum;
        PlotSteadyStateOrig=PlotSteadyState;
        ComputeSteadyStateOrig=ComputeSteadyState;
        StoreStructShocksOrig=StoreStructShocks;
        HistDecompShocksOrig=HistDecompShocks;
        IRHorizon=0;
        FEVDHorizon=[];
        actual_fitted=0;
        crossplot=0;
        actualres=0;
        autocorrres=0;
        histores=0;
        SaveParameters=0;
        estsum=0;
        ComputeSteadyState=0;
        PlotSteadyState=0;
        StoreStructShocks=0;
        HistDecompShocks=cell(0);
        HistDecompShocks{1}=nan;
    end
    
    % Define a string for the data frequency. To be used for printing later.
    switch season
        case 1 % yearly data
            PeriodType='Y';
        case 4 % quarterly data
            PeriodType='Q';
        case 12 % monthly data
            PeriodType='M';
        otherwise
    end
    
    RMSEHorizons=RMSEHorizons(RMSEHorizons<=horizon); % Cannot evaluate RMSE at horizons beyond the forecast horizon. Redefining RMSEHorizons.
    
    
    %%%%%%%%%%%%%%%%%       HERE BEGINS THE GRAND LOOP OVER THE FORECAST DATES       %%%%%%%%%%%%%%%%%%%%%%%%%
    StandCount=0; % This is a counter for sequential forecast dates.
    PeriodCount=0; % This is a counter for the periods within a year, e.g. quarters.
    for estendyear=estendyear1:estendyear2
        if estendyear==estendyear2
            upperperiod=estendperiod2; % Last year of sample, only loop until final period in sample.
        end
        while estendperiod<=upperperiod
            FinalEstDate=[int2str(estendyear),PeriodType,int2str(estendperiod)];
            PeriodCount=PeriodCount+1;
            if PeriodCount>season
                PeriodCount=1;
            end
            
            if estendyear==estendyear2 && estendperiod==upperperiod && SwitchOffDuringSequential% Last year, last period.
                % Retrieve original settings for options that we only do on the longest estimation sample.
                IRHorizon=IRHorizonOrig;
                FEVDHorizon=FEVDHorizonOrig;
                actual_fitted=actual_fittedOrig;
                crossplot=crossplotOrig;
                actualres=actualresOrig;
                autocorrres=autocorrresOrig;
                histores=historesOrig;
                SaveParameters=SaveParametersOrig;
                estsum=estsumOrig;
                PlotSteadyState=PlotSteadyStateOrig;
                ComputeSteadyState=ComputeSteadyStateOrig;
                StoreStructShocks=StoreStructShocksOrig;
                HistDecompShocks=HistDecompShocksOrig;
            end

            tic % Start time clock.
            StandCount=StandCount+1;
            
            % Loads database from Excel file
            bvar_db
            
            if sampledates(end,2)+(sampledates(end,3)-1)/season<estendyear2+(estendperiod2-1)/season+min(RMSEHorizons)/season & PeriodCount==1
                disp('Invalid input: RMSEHorizon. No out-of-sample obs to evaluate even the shortest forecast. RMSEHorizons is set to empty.')
                RMSEHorizons=[];
            end
            % Computing sample and estimation dates from user inputs
            beginest=find(sampledates(:,2)==eststartyear & sampledates(:,3)==eststartperiod);
            endest=find(sampledates(:,2)==estendyear & sampledates(:,3)==estendperiod);
            estdates=sampledates(beginest:endest,2:3);
            estdates=[(1:size(estdates,1))' estdates];
            NoOfOrigVars=size(Z,2);          % This is the total number of endogenous variables in the data base.
            FinalData=Z(beginest:endest,:);  % FinalData contains all endogenous variables in the data base, but only over the est sample. 
            X=Z(beginest:endest,variables);  % X contains only the modelled variables in the data base, and only over the est sample.
            Z=Z(:,variables);                % Z contains only the modelled variables in the data base, but over the full data base sample.
            varlabels=endolabels(variables); % Labels of the modelled endogenous variables.
            MeanFirstLag=MeanFirstLag(variables);
            if StructuralForm==1
                for j=1:length(varlabels)
                    ShockLabels{j}=strcat(varlabels{j},'_s');
                end
            end
                        
            serialdatenumbersest=serialdatenumbers(beginest:endest);
            
            % Dummies that are zero throughout the estimation period are discarded.
            if isempty(D)==0              
                D=D(beginest:endest,dummies);    
                nonzerodummies=sum(abs(D(K+1:end,:)))>0;
                D=D(:,nonzerodummies);
                dummylabels=dummylabels(dummies);
                dummylabels=dummylabels(nonzerodummies);
                domesticdummies=find(isdomesticdummies(nonzerodummies)); % This vector contains the indices of the non-zero dummies.
            else
                dummylabels=cell(0);
                domesticdummies=[];
            end 
            
            q=length(dummylabels); % Number of active dummies
            if IncludeConstant>0
                dummylabels{q+1}='Constant'; % Add constant to the set of exogenous variables.
                q=q+1;
            end
            
            % Printing some model info on the screen
            if StandCount==1
                disp('')
                disp('Endogenous Variables')
                disp(varlabels)
                disp('Prior Mean on First Lag:')            
                disp(MeanFirstLag')
                disp('')
                disp('Dummy/Exogenous')
                disp(dummylabels)
                disp('')
            end
            
            % Extracting info on which variables that are exogenous, in differences, and seasonal.
            exogenous=find(isexogenous(variables));
            difference=find(isdifference(variables));
            SeasonalVars=find(SeasonalVars(variables));

            % Playing around with different date formats.
            OriginalDateLabels=DateLabels;                      % These are of the format Q2-99 ...
            for i=1:size(OriginalDateLabels,1)                  % ... whereas Dates are of the format 1999.25
                if season==12
                    Dates(i,:)=str2num(strcat('19',OriginalDateLabels(i,4:5)))+(str2num(OriginalDateLabels(i,1:2))-1)/season;
                else
                    Dates(i,:)=str2num(strcat('19',OriginalDateLabels(i,4:5)))+(str2num(OriginalDateLabels(i,2))-1)/season;
                end
            end
            
            Dates(Dates<1920)=Dates(Dates<1920)+100;
            T=size(X,1);
            EstDates=Dates(beginest:endest); % These are the estimation dates in the numeric format, 1999.25, 1999.5, 1999.75 2000.00 etc
            
            if IncludeConstant==2 % Mean-adjusted form. Set up prior for deterministic trends.
                % Computing prior mean and standard deviation of deterministic variables
                [muphi,Sigmaphi]=interval_to_moments(intervalcell,LongRunCoverProb,difference,season);
            end
            
            % Checking if the structural VAR is identified
            if StructuralForm==2
                AnsToIdentQuest='yes';
                Identified=identify(Grest);
                if Identified.ord==0
                    AnsToIdentQuest=questdlg('Order condition is not satisfied - Proceed with the estimation?')
                elseif Identified.str1==0
                    AnsToIdentQuest=questdlg('Structure condition is not satisfied - Proceed with the estimation?')
                end
                if strcmpi(AnsToIdentQuest,'yes')==0 % Estimation aborted
                    return;
                end
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%      ESTIMATION and PREDICTION PHASE    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            % For the historical decompositions
            if isnan(HistDecompShocks{1}) % Don't do the decomposition
                ComputeHistDecompShocks=0;
                HistDecompStartObs=0;
            else
                ComputeHistDecompShocks=1;
                if isempty(HistDecompDate)
                    HistDecompStartObs=0;
                else
                    DecompQuarter=eststartperiod+K;
                    PlotStartYear=eststartyear;
                    if DecompQuarter>season
                        PlotStartYear=eststartyear+1;
                        DecompQuarter=DecompQuarter-K;
                    end
                    FirstDateOfDecompData=PlotStartYear+(1/season)*(DecompQuarter-1);
                    HistDecompStartObs=(HistDecompDate-FirstDateOfDecompData)*season+1;
                end
            end
            
            if LoadResultsFromFile==1
                disp('Predictions loaded from file')
            end
            if LoadResultsFromFile==0
                % Generates draws from posterior distribution of model parameters,
                % computes predictions and computes impulse responses (only if StructuralForm>0
                if IncludeConstant<2
                    MeanAdj=0;
                    muphi=nan;
                    Sigmaphi=nan;
                else 
                    MeanAdj=1;
                end
                
                NBurnin=ceil((PercentBurnin/100)*niter);
                if ML==0 % Bayesian estimation.
                    niter=niter+NBurnin;
                else
                    ThinOutFreq=1; % ML
                end
                [Pires,Sigmares,UpsRes,Phires,MeanUpsilon,StdUpsilon,preds,LogPost,NRestrict,MA,HistDecompPreds,EStructCollect]=GibbsBVAR(X,...
                    D,K,niter,NBurnin,ThinOutFreq,difference,exogenous,lambdas,MeanFirstLag,LoadPriorFromFile,MeanAdj,muphi,Sigmaphi,domesticdummies,[],...
                    horizon,IncludeConstant,ML,ComputeLogPost,StructuralForm,Grest,NormVars,WZNorm,DfCorrML,...
                    IRHorizon,varlabels,PriorSeason,SeasonalVars,HistDecompShocks,...
                    HistDecompStartObs,RandSeed,CondHorizon,CondVar,CondShock,CondPath,CondPathInAnnualGrowth,OtherShocks,...
                    FinalEstDate,ForceStationarity,StoreStructShocks);
                
                % Remove burn-in draws
                if ML==0
                    Pires=Pires(NBurnin+1:end,:);
                    Sigmares=Sigmares(NBurnin+1:end,:);
                    UpsRes=UpsRes(NBurnin+1:end,:);
                    Phires=Phires(NBurnin+1:end,:);
                    preds=preds(NBurnin+1:end,:,:);
                    LogPost=LogPost(NBurnin+1:end);
                    MA=MA(:,:,:,floor(NBurnin/ThinOutFreq)+1:end);
                    if ~isnan(HistDecompPreds)
                        HistDecompPreds=HistDecompPreds(floor(NBurnin/ThinOutFreq)+1:end,:,:,:);
                    end

                    if StoreStructShocks
                        EStructCollect=EStructCollect(:,:,floor(NBurnin/ThinOutFreq)+1:end);
                    end

                    niter=niter-NBurnin; % Setting niter back to its original value
                end

            else % load results from file FileName_PeriodType_estendperiod (e.g. ExampleFileName1980Q1.mat)
                load(strcat(SavedResultsFileName,int2str(estendyear),PeriodType,int2str(estendperiod)))
            end
            niter=size(preds,1); % Number of Gibbs iterations
            
            %%%%%%%%%%%%%%% Naive forecasting methods  %%%%%%%%%%%%%%%%%%%%
            NoChangeForecast=repmat(X(end,:),horizon,1);
            MeanForecast=repmat(mean(X(end-NBackObs(StandCount)+1:end,:)),horizon,1); % Forecast is average of the last observations before forecast period.
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%   Marginal likelihood computation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            NModHarmBatches=1;
            Origin=round(niter/NModHarmBatches);
            StepSize=round(niter/NModHarmBatches);
            
            switch IncludeConstant
                case {0,1} % Standard VAR
                    switch LogMargLikeAlgo
                        case 0
                            LogMargLike=nan;
                        case 1 % Geweke's modified harmonic estimator
                            if StructuralForm<2 % Reduced form or recursive
                                LogMargLikes = MargDensModHarmonic([Pires Sigmares],LogPost,[],Origin,StepSize,1);
                            else % Non-recursive
                                LogMargLikes = MargDensModHarmonic([Pires UpsRes],LogPost,[],Origin,StepSize,1);
                            end
                            LogMargLike=mean(LogMargLikes(:,2));
                        case 2 % Chib's estimator
                            LogMargLike=nan; % not yet implemented
                        otherwise
                    end

                case 2 % Steady State VAR
                    switch LogMargLikeAlgo
                        case 0
                            LogMargLike=nan;
                        case 1 % Geweke's modified harmonic estimator
                            if StructuralForm<2 % Reduced form or recursive
                                LogMargLikes = MargDensModHarmonic([Pires Sigmares Phires],LogPost,[],Origin,StepSize,1);
                            else % Non-recursive
                                LogMargLikes = MargDensModHarmonic([Pires UpsRes Phires],LogPost,[],Origin,StepSize,1);
                            end
                            LogMargLike=mean(LogMargLikes(:,2));
                        case 2 % Chib's estimator
                            [LogMargLike,Pires,Phires,Sigmares]=marglikedetmodel(X,D,K,niter,difference,exogenous,lambdas,muphi,Sigmaphi,MeanFirstLag,domesticdummies);
                        otherwise
                    end
                otherwise
            end
            
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%      SUMMARIZING RESULTS AND PLOTTING    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    Impulse Response Graphs   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if StructuralForm>0 && IRHorizon>0
                Endo=[];
                SameScale=0;
                Percent=0;
                FigHandle=figure;
                set(FigHandle,'name','Impulse Response Functions')
                [meanimpulse,upperband,lowerband]=ImpulseSummary(MA,[],1,SameScale,Percent,1,CoverProb,ShockLabels,varlabels,Convert2AnnualIR,difference,season);
                
                % Plotting IRs to a specific shock in a separate graph - uncomment and change if needed.
                % Shock=6; % Plot IR to this shock.
                % Variables=4:7; % Plot IR for these variables.
                % FigHandle=figure;
                % set(FigHandle,'name','Impulse Response Functions - Monetary Policy Shock')
                %  SomeImpulse(meanimpulse,lowerband,upperband,Shock,Variables,IRHorizon,varlabels,ShockLabels)
                
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    Structural Shocks Graphs    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if StoreStructShocks
                [Nrows,Ncols]=ConstructOptimalSubplot(length(variables));
                if size(EStructCollect,3)>1
                    Prctiles=[2.5 50 97.5];
                    PrctileEStructCollect=prctile(EStructCollect,Prctiles,3);
                else
                    PrctileEStructCollect=EStructCollect;
                end
                FigHandle=figure;
                set(FigHandle,'name','Structural Shocks - Posterior Percentiles')
                for VarCount=1:length(variables)
                    subplot(Nrows,Ncols,VarCount)
                    title(ShockLabels{VarCount})
                    hold on
                    plot(EstDates(K+1:end),PrctileEStructCollect(:,VarCount,2),'k','linewidth',2) % Median structural shock
                    if ML>0 % Maximum likelihoood
                        if VarCount==1
                            legend('ML Estimate of Structural Shock')
                        end
                    else % Bayesian estimation
                        plot(EstDates(K+1:end),PrctileEStructCollect(:,VarCount,1),'r') % Lower limit 95% interval for structural shock
                        plot(EstDates(K+1:end),PrctileEStructCollect(:,VarCount,3),'r') % Upper limit 95% interval for structural shock
                        if VarCount==1
                            legend('Posterior Median Structural Shock','95% Intervals of Structural Shock')
                        end
                    end
                    axis tight
                end
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%  Forecast Error Variance Decomposition  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if isempty(FEVDHorizon)
                % Do nothing
            else
                CoverProbFEVD=[100*fliplr((1-CoverProb)/2) 50 100*(1+CoverProb)/2];
                [PercentilesEVD,MeanFEVD]=ForecastErrorVarDecomp(MA,[],[],FEVDHorizon,ShockLabels,varlabels,CoverProbFEVD,1);
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
                        
            % Calling script that transform series and predictions in quarterly/monthly differences to yearly differences
            % Reorganizes preds into predsbyvar so that each cell array contains predictions on one variable
            % e.g. predsbyvar{1} is niter-by-horizon matrix with predictions on the ith variable.
            TransformPredsAndData
            
            % Computes and prints point estimates for parameters. Computes point estimates and intervals for steady states.
            PrctileUnCondMean=EstSummaryMain(estsum,ComputeSteadyState,X,D,Pires,Sigmares,Phires,MeanUpsilon,StdUpsilon,LogMargLike,...
                StructuralForm,K,p,q,lambdas,season,PeriodType,difference,IncludeConstant,ML,estdates,varlabels,dummylabels,MeanFirstLag,OutFileName);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      Steady State Graphs       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if PlotSteadyState
                [Nrows,Ncols]=ConstructOptimalSubplot(length(variables));
                FigHandle=figure;
                set(FigHandle,'name','Steady State - Posterior Percentiles')
                for VarCount=1:length(variables)
                    subplot(Nrows,Ncols,VarCount)
                    title(varlabels{VarCount})
                    hold on
                    plot(EstDates,Xyeardiff(:,VarCount),'k','linewidth',2)
                    plot(EstDates,PrctileUnCondMean(:,VarCount,2),'r') % Median SS
                    if ML>0 % Maximum likelihoood
                        if VarCount==1
                            legend('Data','ML Estimate of Steady State')
                        end
                    else % Bayesian estimation
                        plot(EstDates,PrctileUnCondMean(:,VarCount,1),'r--') % Lower limit 95% interval for SS
                        plot(EstDates,PrctileUnCondMean(:,VarCount,3),'r--') % Upper limit 95% interval for SS
                        if VarCount==1
                            legend('Data','Posterior Median of Steady State','95% Intervals of Steady State')
                        end
                    end
                    axis tight
                end
            end  
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            % Computing naive forecast on annual growth rates
            NoChangeForecastYearDiff=repmat(Xyeardiff(end,:),horizon,1);
            MeanForecastYearDiff=repmat(mean(Xyeardiff(end-NBackObs(StandCount)+1:end,:)),horizon,1); % Forecast is average of the last observations before forecast period.
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      Forecast Path Graphs      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            switch PlotForecast
                case 1
                    if PeriodCount==1 % Current quarter is plotting quarter
                        PlotForecastDynamic=1;
                    else 
                        PlotForecastDynamic=0;
                    end
                case 2
                    PlotForecastDynamic=1; % plots every quarter
                otherwise
                    PlotForecastDynamic=0; % No plotting
            end
            
            % Graphs the unconditional forecasts
            if horizon>0
                PlotYForecastDynamic=1; % Plots every period
                FirstDateOfData=eststartyear+(1/season)*(eststartperiod-1);
                GridOn=1;
                DottedPred=0;
                AllVarsInOneFig=1;
                if isempty(CondShock)
                    GraphTitle=['Unconditional Forecasts - ',FinalEstDate];
                else
                    GraphTitle=['Conditional Forecasts - ',FinalEstDate];
                end
                [MedianYForecast,LowerBoundYForecast,UpperBoundYForecast]=PredSummary(Xyeardiff,preds,FutureData,PlotForecastDynamic,CoverProb,...
                    HPD,FirstDateOfData,season,horizon,lengthofpast,PredVars,PredPlotVars,varlabels,GridOn,DottedPred,AllVarsInOneFig,GraphTitle);
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%      Historical Decomposition Graphs    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if ComputeHistDecompShocks==1
                PlotHistDecomp % Script to plot the historical decomposition.
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%   Computes and Plot Model Fit Diagnostics  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            switch IncludeConstant
                case {0,1}
                    if actual_fitted+crossplot+actualres+autocorrres+histores>0
                        [Fitted,Residual,Median_est_fit,Median_est_residual]=varfitandresidual(XOriginal,D,K,Pires,IncludeConstant,ThinOutFreq);
                    end
                case 2
                    if actual_fitted+crossplot+actualres+autocorrres+histores>0
                        [Fitted,Residual,Median_est_fit,Median_est_residual]=varfitandresidualMeanCorr(XOriginal,D,K,Pires,Phires,...
                            IncludeConstant,ThinOutFreq);
                    end
                otherwise
            end
            
            % Residual diagnostics
            if actual_fitted+crossplot+actualres+autocorrres+histores>0
                XForResidualsAndFit=XOriginal;
                % Converting fit to annual growth rates.
                if Convert2AnnualForFit
                    for j=difference
                        Median_est_fit(:,j)=Diff2AnnualDiff(Median_est_fit(:,j),[],season);
                        for h=1:size(Fitted,3)
                            Fitted(:,j,h)=Diff2AnnualDiff(Fitted(:,j,h),[],season);
                        end
                    end
                    XForFit=Xyeardiff;
                else
                    XForFit=X;
                end
                
                varfitandresidualsummary(Fitted,Residual,Median_est_fit,Median_est_residual,XForFit,D,K,residualplotvars,varlabels,...
                    actual_fitted,crossplot,actualres,autocorrres,nlags,histores,EstMedian,IncludeConstant,EstDates(K+1:end));
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            time=toc;
            
            % Collect the median forecast over the forecast dates.
            MedianYForecastAll(:,:,StandCount)=MedianYForecast;
            
            
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         FORECAST EVALUATION       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            % If post-estimation data has fewer observations that the smallest horizon where the RMSE is evaluated, then end RMSE computations.
            if size(FutureData,1)<min(RMSEHorizons)
                RMSEHorizons=[];
            end
            
            % Collects Forecasts and data for the out-of-sample forecast evaluation.
            for j=1:length(RMSEHorizons)
                if size(FutureData,1)>=RMSEHorizons(j) % Checks if there is data available to evaluate forecast at the given horizon.
                    % PredsFixedHorizon(t,var,h,1), is the RMSEHorizons(h)-step ahead BVAR forecast for variable 'var' made at time t.    
                    PredsFixedHorizon(StandCount,:,j,1)=MedianYForecast(RMSEHorizons(j),PredVars);
                    PredsFixedHorizon(StandCount,:,j,2)=NoChangeForecast(RMSEHorizons(j),PredVars);    
                    PredsFixedHorizon(StandCount,:,j,3)=MeanForecast(RMSEHorizons(j),PredVars);                
                    PredsFixedHorizon(StandCount,:,j,4)=NoChangeForecastYearDiff(RMSEHorizons(j),PredVars);    
                    PredsFixedHorizon(StandCount,:,j,5)=MeanForecastYearDiff(RMSEHorizons(j),PredVars);                
                    DataFixedHorizon(StandCount,:,j)=FutureData(RMSEHorizons(j),PredVars);      
                    
                    if ML==0
                        % Computing log predictive density score based on the normal approximation
                        for u=1:length(MSSESubsetVars)
                            MeanYForecast=mean(preds(:,MSSESubsetVars{u},RMSEHorizons(j)));
                            CovYForecast=cov(preds(:,MSSESubsetVars{u},RMSEHorizons(j)));
                            LogPredScore(StandCount,j,u)=length(MSSESubsetVars{u})*log(2*pi)+log(det(CovYForecast))+...
                                (DataFixedHorizon(StandCount,MSSESubsetVars{u},j)-MeanYForecast)*inv(CovYForecast)*...
                                (DataFixedHorizon(StandCount,MSSESubsetVars{u},j)'-MeanYForecast');
                        end

                        % Analysis of forecast interval coverage for BVAR forecasts
                        for CoverProbCount=1:NCoverProb
                            Hit=DataFixedHorizon(StandCount,:,j)>=LowerBoundYForecast(RMSEHorizons(j),:,CoverProbCount) & DataFixedHorizon(StandCount,:,j)<=UpperBoundYForecast(RMSEHorizons(j),:,CoverProbCount);
                            BandCoverage(StandCount,:,j,CoverProbCount)=999*(Hit==1)-999*(Hit==0); %  999 means interval covers realization
                            % -999 means interval does no cover realization.
                        end %    0 means forecast horizon exceeds remaining out-of-sample data
                    end
                end
            end
            
            if SaveParameters==1
                save(strcat(OutFileName,int2str(estendyear),PeriodType,int2str(estendperiod)),'varlabels','dummylabels','X','D','Xyeardiff','K','lambdas','Pires','Phires','Sigmares','preds','time','LogPost','LogMargLike','MedianYForecast','MeanUnCondMean','MeanYForecast','CovYForecast')
            else
                save(strcat(OutFileName,int2str(estendyear),PeriodType,int2str(estendperiod)),'varlabels','dummylabels','X','D','Xyeardiff','K','lambdas','preds','time','LogMargLike','MedianYForecast','MeanUnCondMean','MeanYForecast','CovYForecast')
            end
            
            estendperiod=estendperiod+1;
        end % Ends loop over final estimation quarter
        estendperiod=1;
    end % Ends loop over final estimation year
        
    save(OutFileName,'varlabels','dummylabels','K','lambdas','LogMargLike','MedianYForecastAll','PredsFixedHorizon','DataFixedHorizon')
    
    if exist('PredsFixedHorizon')
        if isempty(RMSEHorizons)==0
            RMSEHorizons=RMSEHorizons(1:size(PredsFixedHorizon,3)); % Removes the RMSEHorizons where no data is available to evaluate the forecasts
        else
            return;
        end
        if size(PredsFixedHorizon(:,:,1,1),1)<=1 % At least two rolling periods needed to compute RMSEs.
            return;
        end
    end

    ShortVarLabels=endolabels(variables);
    FinalData=Z;
    FinalData=[Dates FinalData]; % FinalData contains (dates in first column), and then annualized data, but all observations in data base.
    ConvertVars=difference;
    
    if length(RMSEHorizons)>0
        EvalHorizons=RMSEHorizons; % Change this if a subset (sic) of RMSEHorizons is wanted in the print out.
        ComputeLogPredScore=1; % Determines whether or not log predictive density score is computed for the predictive distributions
        ComputeBandCoverage=1; % Determines whether or not empirical coverage probs are computed for the forecast bands
        
        ModelLabels{1}='BVAR'; % Name of BVAR model
        CombinedNaive=1; %  If=1, also a combined naive forecast is computed with is NoChange for vars in levels and RecentMean for vars in growth rates.
        PrintHorizons=RMSEHorizons;
        PrintVars=PredVars; % Prints results for all variables
        PrintModels=[1:5];
        PrintMSSESubsets=1:length(MSSESubsetVars); % Which subsets should we print multivariate accuracy results for?
        
        ModelLabels{2}='NoChange';
        ModelLabels{3}='RecentMean';
        ModelLabels{4}='NoChange Yearly';
        ModelLabels{5}='RecentMean Yearly';      
        
        %%%%%%%%%%%%%%%%%%%%%%%%% Computes and prints a range of out-of-sample forecasting precision measures, such as RMSE and multivariate log determinant, from the rolling forecasts.
        
        % Computes the Scale matrix for the multivariate (MSSE) accuracy measures.
        if strcmpi(ScaleMatrixFile,'stdev')
            % Compute scale matrix from data FirstDateForScale:Present.
            FirstYearForScale=str2num(FirstDateForScale(1:4));
            FirstQuarterForScale=str2num(FirstDateForScale(6));
            FirstPeriodForScale=find(FinalData(:,1)==FirstYearForScale+(FirstQuarterForScale-1)/4);
            MSSEScaleMatrix=inv(diag(var(FinalData(FirstPeriodForScale:end,PredVars+1)))); % MSSE matrix is scaled by the standard deviations of the predicted time series.
        elseif strcmpi(ScaleMatrixFile,'identity') | strcmpi(ScaleMatrixFile,'none')
            MSSEScaleMatrix=eye(length(PredVars),length(PredVars));
        else
            % Load scale matrix from file
            load(ScaleMatrixFile)
        end
                
        EvalHorizons=intersect(RMSEHorizons,EvalHorizons);
        PrintHorizons=intersect(EvalHorizons,PrintHorizons);
        
        % Measuring the accuracy of point predictions
        [RMSE,MAE,MAPE,MPE,MSSE,LogDetMSSE,TraceMSSE]=PointPredAccuracy(PredsFixedHorizon,DataFixedHorizon,EvalHorizons,MSSESubsetVars,MSSEScaleMatrix);
        PrintPointPredEval(RMSE,MAE,MAPE,MPE,LogDetMSSE,TraceMSSE,MSSESubsetVars,ShortVarLabels,ModelLabels,EvalHorizons,PrintHorizons,PrintMeasures,PrintVars,PrintModels,PrintMSSESubsets,strcat(OutFileName,'_PointPred'));

        % Measuring the accuracy of predictive densities
        if ML==0
            [MeanLogPredScore,BandCoverageProbs,PostProbCoverage]=DistPredAccuracy(LogPredScore,BandCoverage,MSSESubsetVars,EvalHorizons,CoverProb);
            PrintDistPredEval(MeanLogPredScore,PostProbCoverage,BandCoverageProbs,ModelLabels{1},MSSESubsetVars,PrintVars,EvalHorizons,ShortVarLabels,CoverProb,strcat(OutFileName,'_DistPred'));
        end
    end

    % Plotting all sequential predictions along with actual data in a single graph (Cascade plots)
    if ~isempty(CascadePlotVars)
        LengthOfPastCascade=12;
        FirstForecastDate=estendyear1+(estendperiod1-1)/season+(1/season);
        GraphTitle=['Cascade Plot for ',ModelLabels{1}];
        CascadeForecastPlot(MedianYForecastAll,Xyeardiff,EstDates,CascadePlotVars,varlabels(PredVars),horizon,LengthOfPastCascade,FirstForecastDate,season,CascadeSubPlots,GraphTitle)
    end
    
    save(OutFileName,'PredsFixedHorizon','DataFixedHorizon','EvalHorizons','ModelLabels','MSSESubsetVars','MSSEScaleMatrix','CoverProb','RMSE','MAE','MAPE','MPE','MSSE','LogDetMSSE','TraceMSSE','MeanLogPredScore','BandCoverageProbs','PostProbCoverage','-append')

end % Ends loop over different lambdas or lag lengths
