function [DMTest,MeanDiffPredLoss,ARMASelLog]=PredTest(PredErr1,PredErr2,InputArg)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:      Computes various test for comparing predictive accuracy of two models:
%
%               - Diebold-Mariano (1995, JBES)
%
% INPUT:        PredErr1            (NHoldOutObs-by-1)       Vector of forecast errors in the hold-out sample for the first model.
%               PredErr2            (NHoldOutObs-by-1)       Vector of forecast errors in the hold-out sample for the second model.
%               PredHorizon         (Scalar)                 Forecast horizon
%               InputArg            (Structure) 
%                                   .DMLoss                  Loss function in the DM-test. Choices: 'Quad', 'Abs'.
%                                   .LagTrunc                Method used to truncate the lag length in the spectral density estimate. 
%                                                            Choices: 'Log','Rect'.
%                                   .PredHorizon             Forecast horizon. Only needed if SpectralEst='Trunc'.
%                                   .SpectralEst             Choices: 'Trunc', 'ARMA'
%
% OUTPUT:       DMTest              (scalar)                 The Diebold-Mariano statistic. Follows the N(0,1) under the null.
%               MeanDiffPredLoss    (scalar)                 Mean of the difference of d(t)=g[e1(t)]-g[e2(t)], where g(.) is the loss function.
% 
% AUTHOR:       Mattias Villani, Research Department, Sveriges Riksbank and
%               Department of Statistics, Stockholm University. 
%               E-mail: mattias.villani@riksbank.se
%   
% FIRST VER.:   2005-04-19
% REVISED:      2005-04-19
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T=length(PredErr1); % Number of out-of-sample obs
ARMASelLog=nan;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Diebold-Mariano (1995, JBES) test %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computing loss of forecasts errors
switch InputArg.DMLoss
    case {'Quad','quad'}
        PredLoss1=PredErr1.^2; 
        PredLoss2=PredErr2.^2;
    case {'Abs','abs'}
        PredLoss1=abs(PredErr1);
        PredLoss2=abs(PredErr2);
    otherwise
        % Do all losses
end

DiffPredLoss=PredLoss1-PredLoss2; % d(t) in DM's notation
MeanDiffPredLoss=mean(DiffPredLoss); % dbar in DM's notation.

% Selecting Spectral Density Estimator (zero frequency)
switch InputArg.SpectralEst
    case {'Trunc','trunc'} % Truncation of autocovariances
        % Computing lag truncation
        switch InputArg.LagTrunc
            case {'Log','log'}
                NLags=round(log(T)); 
            case {'Rect','rect'}
                NLags=InputArg.PredHorizon-1;
            otherwise
        end
        ACFDiffPredLoss = xcorr(DiffPredLoss,NLags,'biased'); % Autocovariances for k=-MaxLag,...,0,...,MaxLag.
        SpecDensFreqZero=(1/(2*pi))*sum(ACFDiffPredLoss);
    case {'arma','ARMA'} % ARMA-based spectral density estimate
        SearchAR=0:T-1;
        SearchMA=0:T-1;
        SearchARMA=0:T-1;
        [ar_est ma_est ARMASelLog] = armasel(DiffPredLoss-mean(DiffPredLoss),SearchAR,SearchMA,SearchARMA); % Find optimal ARMA model
        [SpectralDensity,XGrid]=arma2psd(ar_est,ma_est); % Compute the spectrum
        SpecDensFreqZero=SpectralDensity(1); % Computes the spectral density at the zero frequency
    otherwise
end

DMTest=MeanDiffPredLoss/sqrt(2*pi*SpecDensFreqZero/T);
